/*  CPfront 1.0 -48 */
#include "SYSTEM.h"
#include "Kernel.h"
#include "PosixCtypes.h"
#include "stdlib.h"
#include "locale.h"
#include "iconv.h"
#include "sys/types.h"
#include "errno.h"
#include "PosixCmacro.h"
#include "sys/stat.h"
#include "time.h"


export CHAR HostLang_lang[32], HostLang_country[32], HostLang_encoding[32];
static iconv_t HostLang_c2sc, HostLang_sc2c, HostLang_invalid;

export INTEGER HostLang__1__desc[];
export SYSTEM_TYPEDESC *HostLang__1__typ = (SYSTEM_TYPEDESC*)HostLang__1__desc;

export void HostLang_HostToString (SHORTCHAR *in, INTEGER in__len, CHAR *out, INTEGER out__len, INTEGER \
mode, INTEGER *res);
static void HostLang_Init (void);
export void HostLang_StringToHost (CHAR *in, INTEGER in__len, SHORTCHAR *out, INTEGER out__len, INTEGER \
mode, INTEGER *res);

export void HostLang__reg();
export void HostLang__body();
export struct SYSTEM_MODDESC HostLang__desc;


void HostLang_StringToHost (CHAR *in, INTEGER in__len, SHORTCHAR *out, INTEGER out__len, INTEGER mode, \
INTEGER *res)
{
	INTEGER i, j, err;
	size_t maxlen, len, count;
	Kernel_ADDRESS inadr, outadr;
	SHORTCHAR sch;
	__ENTER("HostLang.StringToHost");
	__ASSERT(__IN(mode, 0x07), 20);
	__ASSERT(HostLang_c2sc != HostLang_invalid, 100);
	i = 0;
	err = 0;
	outadr = (INTEGER)&out[0];
	maxlen = out__len - 1;
	while (err == 0 && in[__X(i, in__len)] != 0) {
		j = i;
		while (in[__X(i, in__len)] != 0 && (in[__X(i, in__len)] < 55296 || in[__X(i, in__len)] > 55551)) {
			i += 1;
		}
		len = __ASHL(i - j, 1, INTEGER);
		while ((err == 0 && len > 0) && maxlen > 0) {
			inadr = (INTEGER)&in[__X(j, in__len)];
			count = iconv(HostLang_c2sc, (void*)&inadr, &len, (void*)&outadr, &maxlen);
			if (count != 0) {
				switch (PosixCmacro_errno()) {
					case 84: 
						if (mode == 0) {
							if (maxlen < 1) {
								err = 1;
							} else {
								sch = (SHORTCHAR)'\?';
								__PUT(outadr, sch, SHORTCHAR);
								outadr += 1;
								maxlen -= 1;
								j += 1;
								len -= 2;
							}
						} else {
							err = 3;
						}
						break;
					case 7: 
						err = 1;
						break;
					case 22: 
						__HALT(101);
						break;
					case 9: 
						__HALT(102);
						break;
					default: 
						__HALT(103);
						break;
				}
			}
		}
		while ((err == 0 && in[__X(i, in__len)] >= 55296) && in[__X(i, in__len)] <= 55551) {
			if (maxlen < 1) {
				err = 1;
			} else {
				sch = (SHORTCHAR)__MASK((INTEGER)in[__X(i, in__len)], -256);
				if (__IN(mode, 0x03) || sch > 127) {
					__PUT(outadr, sch, SHORTCHAR);
					outadr += 1;
					maxlen -= 1;
					i += 1;
				} else {
					err = 3;
				}
			}
		}
	}
	sch = 0;
	__PUT(outadr, sch, SHORTCHAR);
	*res = err;
	__EXIT;
}

void HostLang_HostToString (SHORTCHAR *in, INTEGER in__len, CHAR *out, INTEGER out__len, INTEGER mode, \
INTEGER *res)
{
	INTEGER err;
	size_t maxin, maxout, count;
	Kernel_ADDRESS inadr, outadr;
	SHORTCHAR sch;
	CHAR ch;
	__ENTER("HostLang.HostToString");
	__ASSERT(__IN(mode, 0x07), 20);
	__ASSERT(HostLang_sc2c != HostLang_invalid, 100);
	err = 0;
	inadr = (INTEGER)&in[0];
	outadr = (INTEGER)&out[0];
	maxin = __STRLENS(in);
	maxout = __ASHL(out__len, 1, INTEGER) - 2;
	while ((err == 0 && maxout > 1) && maxin > 0) {
		count = iconv(HostLang_sc2c, (void*)&inadr, &maxin, (void*)&outadr, &maxout);
		if (count != 0) {
			switch (PosixCmacro_errno()) {
				case 7: case 22: case 84: 
					if (maxout < 2) {
						err = 1;
					} else if (maxin < 1) {
						err = 2;
					} else {
						if (mode == 0) {
							sch = (SHORTCHAR)'\?';
							ch = (CHAR)L'\?';
							inadr += 1;
							maxin -= 1;
						} else {
							__GET(inadr, sch, SHORTCHAR);
							inadr += 1;
							maxin -= 1;
							ch = (CHAR)(55296 + (INTEGER)sch);
						}
						if (__IN(mode, 0x03) || sch > 127) {
							__PUT(outadr, ch, CHAR);
							outadr += 2;
							maxout -= 2;
						} else {
							err = 3;
						}
					}
					break;
				case 9: 
					__HALT(101);
					break;
				default: 
					__HALT(102);
					break;
			}
		}
	}
	ch = 0;
	__PUT(outadr, ch, CHAR);
	*res = err;
	__EXIT;
}

static void HostLang_Init (void)
{
	SHORTCHAR *p = NIL;
	INTEGER i, j;
	SHORTCHAR enc[32];
	__ENTER("HostLang.Init");
	HostLang_invalid = (iconv_t)-1;
	p = setlocale(6, (void*)"");
	if (p == NIL) {
		p = getenv((void*)"LANG");
		if (p == NIL) {
			p = "";
		}
	}
	if (((p)[0] == 0 || __STRCMPSS(p, "C") == 0) || __STRCMPSS(p, "POSIX") == 0) {
		HostLang_lang[0] = 0;
		HostLang_country[0] = 0;
		enc[0] = 0;
	} else {
		i = 0;
		j = 0;
		while (((j < 31 && (p)[i] != 0) && (p)[i] != (SHORTCHAR)'_') && (p)[i] != (SHORTCHAR)'.') {
			HostLang_lang[__X(j, 32)] = (p)[i];
			i += 1;
			j += 1;
		}
		HostLang_lang[__X(j, 32)] = 0;
		if (j >= 31) {
			while (((p)[i] != 0 && (p)[i] != (SHORTCHAR)'_') && (p)[i] != (SHORTCHAR)'.') {
				i += 1;
			}
			HostLang_lang[0] = 0;
		}
		if ((p)[i] == (SHORTCHAR)'_') {
			i += 1;
			j = 0;
			while ((j < 31 && (p)[i] != 0) && (p)[i] != (SHORTCHAR)'.') {
				HostLang_country[__X(j, 32)] = (p)[i];
				i += 1;
				j += 1;
			}
			HostLang_country[__X(j, 32)] = 0;
			if (j >= 31) {
				while ((p)[i] != 0 && (p)[i] != (SHORTCHAR)'.') {
					i += 1;
				}
				HostLang_country[0] = 0;
			}
		}
		if ((p)[i] == (SHORTCHAR)'.') {
			i += 1;
			j = 0;
			while (j < 31 && (p)[i] != 0) {
				enc[__X(j, 32)] = (p)[i];
				i += 1;
				j += 1;
			}
			enc[__X(j, 32)] = 0;
			if (j >= 31) {
				enc[0] = 0;
			}
		}
	}
	HostLang_sc2c = HostLang_invalid;
	HostLang_c2sc = HostLang_invalid;
	HostLang_sc2c = iconv_open((void*)"UCS-2LE", (void*)enc);
	if (HostLang_sc2c == HostLang_invalid) {
		__MOVE("ASCII", enc, 6);
		HostLang_sc2c = iconv_open((void*)"UCS-2LE", (void*)enc);
		__ASSERT(HostLang_sc2c != HostLang_invalid, 100);
	}
	HostLang_c2sc = iconv_open((void*)enc, (void*)"UCS-2LE");
	__ASSERT(HostLang_c2sc != HostLang_invalid, 101);
	__STRCOPYSL(enc, HostLang_encoding, 32);
	__EXIT;
}

export INTEGER HostLang__1__desc[] = {
	32,
	(INTEGER)&HostLang__desc,
	0<<8 | 0x02,
	3
};
static SYSTEM_MODDESC *HostLang__imp[] = {
	&Kernel__desc,
	&PosixCmacro__desc,
};
static INTEGER HostLang__exp[] = {
	12,
	0xb36f6ef3, (INTEGER)HostLang_HostToString, 1<<8 | 0x44, 0,
	0x227d6f36, (INTEGER)HostLang_Init, 14<<8 | 0x14, 0,
	0x1c0375bb, (INTEGER)HostLang_StringToHost, 19<<8 | 0x44, 0,
	0x1ee54c0a, (INTEGER)&HostLang_c2sc, 32<<8 | 0x13, 6,
	0xdd59ed88, (INTEGER)&HostLang_country, 37<<8 | 0x23, (INTEGER)HostLang__1__desc,
	0xdd59ed88, (INTEGER)&HostLang_encoding, 45<<8 | 0x23, (INTEGER)HostLang__1__desc,
	0x1ee54c0a, (INTEGER)&HostLang_invalid, 54<<8 | 0x13, 6,
	0xdd59ed88, (INTEGER)&HostLang_lang, 62<<8 | 0x23, (INTEGER)HostLang__1__desc,
	0x5fcf3f34, 0, 67<<8 | 0x41, 0,
	0x528c19ed, 0, 74<<8 | 0x41, 0,
	0x564d045a, 0, 82<<8 | 0x41, 0,
	0x1ee54c0a, (INTEGER)&HostLang_sc2c, 90<<8 | 0x13, 6,
};
static char HostLang__names[] = {
	0,
	'H','o','s','t','T','o','S','t','r','i','n','g',0,
	'I','n','i','t',0,
	'S','t','r','i','n','g','T','o','H','o','s','t',0,
	'c','2','s','c',0,
	'c','o','u','n','t','r','y',0,
	'e','n','c','o','d','i','n','g',0,
	'i','n','v','a','l','i','d',0,
	'l','a','n','g',0,
	'p','e','p','3','8','3',0,
	'p','e','p','3','8','3','x',0,
	'r','e','p','l','a','c','e',0,
	's','c','2','c',0,
};
static INTEGER HostLang__ptrs[] = {
	0
};
struct SYSTEM_MODDESC HostLang__desc = {
	0, 13, 0, /* next, opts, refcnt */ 
	{2019, 7, 23, 0, 2, 27}, /* compTime */ 
	{0, 0, 0, 0, 0, 0}, /* loadTime */ 
	HostLang__body,
	0,
	2, /* nofimps */ 
	0, /* nofptrs */ 
	0, 0, 0, 0, 0, 0, 0, 0, /* csize..varBase */ 
	HostLang__names,
	HostLang__ptrs,
	HostLang__imp,
	(SYSTEM_DIRECTORY*)HostLang__exp,
	"HostLang"
};

export void HostLang__reg()
{
	__BEGREG(HostLang__desc);
	Kernel__reg();
	PosixCmacro__reg();
	__REGMOD(HostLang__desc);
	__ENDREG;
}

export void HostLang__body()
{
	__BEGBODY(HostLang__desc);
	Kernel__body();
	PosixCmacro__body();
	HostLang_Init();
	__ENDBODY;
}
